@@ -20,10 +20,12 @@
 
 import bpy
 from bpy.types import Operator
-from bpy.props import (StringProperty,
-                       BoolProperty,
-                       EnumProperty,
-                       IntProperty)
+from bpy.props import (
+    BoolProperty,
+    EnumProperty,
+    IntProperty,
+    StringProperty,
+)
 
 
 class SelectPattern(Operator):
@@ -32,23 +34,23 @@ class SelectPattern(Operator):
     bl_label = "Select Pattern"
     bl_options = {'REGISTER', 'UNDO'}
 
-    pattern = StringProperty(
-            name="Pattern",
-            description="Name filter using '*', '?' and "
-                        "'[abc]' unix style wildcards",
-            maxlen=64,
-            default="*",
-            )
-    case_sensitive = BoolProperty(
-            name="Case Sensitive",
-            description="Do a case sensitive compare",
-            default=False,
-            )
-    extend = BoolProperty(
-            name="Extend",
-            description="Extend the existing selection",
-            default=True,
-            )
+    pattern: StringProperty(
+        name="Pattern",
+        description="Name filter using '*', '?' and "
+        "'[abc]' unix style wildcards",
+        maxlen=64,
+        default="*",
+    )
+    case_sensitive: BoolProperty(
+        name="Case Sensitive",
+        description="Do a case sensitive compare",
+        default=False,
+    )
+    extend: BoolProperty(
+        name="Extend",
+        description="Extend the existing selection",
+        default=True,
+    )
 
     def execute(self, context):
 
@@ -60,11 +62,13 @@ class SelectPattern(Operator):
             pattern_match = (lambda a, b:
                              fnmatch.fnmatchcase(a.upper(), b.upper()))
         is_ebone = False
+        is_pbone = False
         obj = context.object
         if obj and obj.mode == 'POSE':
             items = obj.data.bones
             if not self.extend:
                 bpy.ops.pose.select_all(action='DESELECT')
+            is_pbone = True
         elif obj and obj.type == 'ARMATURE' and obj.mode == 'EDIT':
             items = obj.data.edit_bones
             if not self.extend:
@@ -75,19 +79,23 @@ class SelectPattern(Operator):
             if not self.extend:
                 bpy.ops.object.select_all(action='DESELECT')
 
-        # Can be pose bones or objects
+        # Can be pose bones, edit bones or objects
         for item in items:
             if pattern_match(item.name, self.pattern):
-                item.select = True
 
                 # hrmf, perhaps there should be a utility function for this.
                 if is_ebone:
+                    item.select = True
                     item.select_head = True
                     item.select_tail = True
                     if item.use_connect:
                         item_parent = item.parent
                         if item_parent is not None:
                             item_parent.select_tail = True
+                elif is_pbone:
+                    item.select = True
+                else:
+                    item.select_set(True)
 
         return {'FINISHED'}
 
@@ -95,7 +103,7 @@ class SelectPattern(Operator):
         wm = context.window_manager
         return wm.invoke_props_popup(self, event)
 
-    def draw(self, context):
+    def draw(self, _context):
         layout = self.layout
 
         layout.prop(self, "pattern")
@@ -103,6 +111,11 @@ class SelectPattern(Operator):
         row.prop(self, "case_sensitive")
         row.prop(self, "extend")
 
+    @classmethod
+    def poll(cls, context):
+        obj = context.object
+        return (not obj) or (obj.mode == 'OBJECT') or (obj.type == 'ARMATURE')
+
 
 class SelectCamera(Operator):
     """Select the active camera"""
@@ -110,10 +123,17 @@ class SelectCamera(Operator):
     bl_label = "Select Camera"
     bl_options = {'REGISTER', 'UNDO'}
 
+    extend: BoolProperty(
+        name="Extend",
+        description="Extend the selection",
+        default=False,
+    )
+
     def execute(self, context):
         scene = context.scene
+        view_layer = context.view_layer
         view = context.space_data
-        if view.type == 'VIEW_3D' and not view.lock_camera_and_layers:
+        if view.type == 'VIEW_3D' and view.use_local_camera:
             camera = view.camera
         else:
             camera = scene.camera
@@ -123,8 +143,11 @@ class SelectCamera(Operator):
         elif camera.name not in scene.objects:
             self.report({'WARNING'}, "Active camera is not in this scene")
         else:
-            context.scene.objects.active = camera
-            camera.select = True
+            if not self.extend:
+                bpy.ops.object.select_all(action='DESELECT')
+            view_layer.objects.active = camera
+            # camera.hide = False  # XXX TODO where is this now?
+            camera.select_set(True)
             return {'FINISHED'}
 
         return {'CANCELLED'}
@@ -132,30 +155,32 @@ class SelectCamera(Operator):
 
 class SelectHierarchy(Operator):
     """Select object relative to the active object's position """ \
-    """in the hierarchy"""
+        """in the hierarchy"""
     bl_idname = "object.select_hierarchy"
     bl_label = "Select Hierarchy"
     bl_options = {'REGISTER', 'UNDO'}
 
-    direction = EnumProperty(
-            items=(('PARENT', "Parent", ""),
-                   ('CHILD', "Child", ""),
-                   ),
-            name="Direction",
-            description="Direction to select in the hierarchy",
-            default='PARENT')
-
-    extend = BoolProperty(
-            name="Extend",
-            description="Extend the existing selection",
-            default=False,
-            )
+    direction: EnumProperty(
+        items=(
+            ('PARENT', "Parent", ""),
+            ('CHILD', "Child", ""),
+        ),
+        name="Direction",
+        description="Direction to select in the hierarchy",
+        default='PARENT',
+    )
+    extend: BoolProperty(
+        name="Extend",
+        description="Extend the existing selection",
+        default=False,
+    )
 
     @classmethod
     def poll(cls, context):
         return context.object
 
     def execute(self, context):
+        view_layer = context.view_layer
         select_new = []
         act_new = None
 
@@ -169,7 +194,7 @@ class SelectHierarchy(Operator):
             for obj in selected_objects:
                 parent = obj.parent
 
-                if parent:
+                if parent and parent.visible_get():
                     if obj_act == obj:
                         act_new = parent
 
@@ -177,7 +202,7 @@ class SelectHierarchy(Operator):
 
         else:
             for obj in selected_objects:
-                select_new.extend(obj.children)
+                select_new.extend([child for child in obj.children if child.visible_get()])
 
             if select_new:
                 select_new.sort(key=lambda obj_iter: obj_iter.name)
@@ -189,34 +214,33 @@ class SelectHierarchy(Operator):
                 bpy.ops.object.select_all(action='DESELECT')
 
             for obj in select_new:
-                obj.select = True
+                obj.select_set(True)
 
-            context.scene.objects.active = act_new
+            view_layer.objects.active = act_new
             return {'FINISHED'}
 
         return {'CANCELLED'}
 
 
 class SubdivisionSet(Operator):
-    """Sets a Subdivision Surface Level (1-5)"""
+    """Sets a Subdivision Surface level (1 to 5)"""
 
     bl_idname = "object.subdivision_set"
     bl_label = "Subdivision Set"
     bl_options = {'REGISTER', 'UNDO'}
 
-    level = IntProperty(
-            name="Level",
-            min=-100, max=100,
-            soft_min=-6, soft_max=6,
-            default=1,
-            )
-
-    relative = BoolProperty(
-            name="Relative",
-            description=("Apply the subsurf level as an offset "
-                         "relative to the current level"),
-            default=False,
-            )
+    level: IntProperty(
+        name="Level",
+        min=-100, max=100,
+        soft_min=-6, soft_max=6,
+        default=1,
+    )
+    relative: BoolProperty(
+        name="Relative",
+        description=("Apply the subdivision surface level as an offset "
+                     "relative to the current level"),
+        default=False,
+    )
 
     @classmethod
     def poll(cls, context):
@@ -237,13 +261,17 @@ class SubdivisionSet(Operator):
             for mod in obj.modifiers:
                 if mod.type == 'MULTIRES':
                     if not relative:
-                        if level <= mod.total_levels:
-                            if obj.mode == 'SCULPT':
-                                if mod.sculpt_levels != level:
-                                    mod.sculpt_levels = level
-                            elif obj.mode == 'OBJECT':
-                                if mod.levels != level:
-                                    mod.levels = level
+                        if level > mod.total_levels:
+                            sub = level - mod.total_levels
+                            for _ in range(sub):
+                                bpy.ops.object.multires_subdivide(modifier="Multires")
+
+                        if obj.mode == 'SCULPT':
+                            if mod.sculpt_levels != level:
+                                mod.sculpt_levels = level
+                        elif obj.mode == 'OBJECT':
+                            if mod.levels != level:
+                                mod.levels = level
                         return
                     else:
                         if obj.mode == 'SCULPT':
@@ -265,8 +293,14 @@ class SubdivisionSet(Operator):
 
             # add a new modifier
             try:
-                mod = obj.modifiers.new("Subsurf", 'SUBSURF')
-                mod.levels = level
+                if obj.mode == 'SCULPT':
+                    mod = obj.modifiers.new("Multires", 'MULTIRES')
+                    if level > 0:
+                        for _ in range(level):
+                            bpy.ops.object.multires_subdivide(modifier="Multires")
+                else:
+                    mod = obj.modifiers.new("Subdivision", 'SUBSURF')
+                    mod.levels = level
             except:
                 self.report({'WARNING'},
                             "Modifiers cannot be added to object: " + obj.name)
@@ -278,37 +312,37 @@ class SubdivisionSet(Operator):
 
 
 class ShapeTransfer(Operator):
-    """Copy another selected objects active shape to this one by """ \
-    """applying the relative offsets"""
+    """Copy the active shape key of another selected object to this one"""
 
     bl_idname = "object.shape_key_transfer"
     bl_label = "Transfer Shape Key"
     bl_options = {'REGISTER', 'UNDO'}
 
-    mode = EnumProperty(
-            items=(('OFFSET',
-                    "Offset",
-                    "Apply the relative positional offset",
-                    ),
-                   ('RELATIVE_FACE',
-                    "Relative Face",
-                    "Calculate relative position (using faces)",
-                    ),
-                   ('RELATIVE_EDGE',
-                   "Relative Edge",
-                   "Calculate relative position (using edges)",
-                   ),
-                   ),
-            name="Transformation Mode",
-            description="Relative shape positions to the new shape method",
-            default='OFFSET',
-            )
-    use_clamp = BoolProperty(
-            name="Clamp Offset",
-            description=("Clamp the transformation to the distance each "
-                         "vertex moves in the original shape"),
-            default=False,
-            )
+    mode: EnumProperty(
+        items=(
+            ('OFFSET',
+             "Offset",
+             "Apply the relative positional offset",
+             ),
+            ('RELATIVE_FACE',
+             "Relative Face",
+             "Calculate relative position (using faces)",
+             ),
+            ('RELATIVE_EDGE',
+             "Relative Edge",
+             "Calculate relative position (using edges)",
+             ),
+        ),
+        name="Transformation Mode",
+        description="Relative shape positions to the new shape method",
+        default='OFFSET',
+    )
+    use_clamp: BoolProperty(
+        name="Clamp Offset",
+        description=("Clamp the transformation to the distance each "
+                     "vertex moves in the original shape"),
+        default=False,
+    )
 
     def _main(self, ob_act, objects, mode='OFFSET', use_clamp=False):
 
@@ -341,10 +375,15 @@ class ShapeTransfer(Operator):
 
         orig_normals = me_nos(me.vertices)
         # actual mesh vertex location isn't as reliable as the base shape :S
-        #~ orig_coords = me_cos(me.vertices)
+        # orig_coords = me_cos(me.vertices)
         orig_coords = me_cos(me.shape_keys.key_blocks[0].data)
 
         for ob_other in objects:
+            if ob_other.type != 'MESH':
+                self.report({'WARNING'},
+                            ("Skipping '%s', "
+                             "not a mesh") % ob_other.name)
+                continue
             me_other = ob_other.data
             if len(me_other.vertices) != len(me.vertices):
                 self.report({'WARNING'},
@@ -373,11 +412,8 @@ class ShapeTransfer(Operator):
                                     (orig_shape_coords[i] - orig_coords[i]))
 
             elif mode == 'RELATIVE_FACE':
-                loops_vidxs = me.loops.foreach_get("vert_index")
                 for poly in me.polygons:
-                    l_start = l_stop = poly.loop_start
-                    l_stop += poly.loop_total
-                    idxs = loops_vidxs[l_start:l_stop]
+                    idxs = poly.vertices[:]
                     v_before = idxs[-2]
                     v = idxs[-1]
                     for v_after in idxs:
@@ -448,7 +484,7 @@ class ShapeTransfer(Operator):
         objects = [ob for ob in context.selected_editable_objects
                    if ob != ob_act]
 
-        if 1:  # swap from/to, means we cant copy to many at once.
+        if 1:  # swap from/to, means we can't copy to many at once.
             if len(objects) != 1:
                 self.report({'ERROR'},
                             ("Expected one other selected "
@@ -468,9 +504,11 @@ class ShapeTransfer(Operator):
 
 
 class JoinUVs(Operator):
-    """Copy UV Layout to objects with matching geometry"""
+    """Transfer UV Maps from active to selected objects """ \
+        """(needs matching geometry)"""
     bl_idname = "object.join_uvs"
-    bl_label = "Join as UVs"
+    bl_label = "Transfer UV Maps"
+    bl_options = {'REGISTER', 'UNDO'}
 
     @classmethod
     def poll(cls, context):
@@ -486,7 +524,7 @@ class JoinUVs(Operator):
         if is_editmode:
             bpy.ops.object.mode_set(mode='OBJECT', toggle=False)
 
-        if not mesh.uv_textures:
+        if not mesh.uv_layers:
             self.report({'WARNING'},
                         "Object: %s, Mesh: '%s' has no UVs"
                         % (obj.name, mesh.name))
@@ -524,11 +562,11 @@ class JoinUVs(Operator):
                             else:
                                 uv_other = mesh_other.uv_layers.active
                                 if not uv_other:
-                                    mesh_other.uv_textures.new()
+                                    mesh_other.uv_layers.new()
                                     uv_other = mesh_other.uv_layers.active
                                     if not uv_other:
                                         self.report({'ERROR'}, "Could not add "
-                                                    "a new UV map tp object "
+                                                    "a new UV map to object "
                                                     "'%s' (Mesh '%s')\n"
                                                     % (obj_other.name,
                                                        mesh_other.name,
@@ -537,6 +575,7 @@ class JoinUVs(Operator):
 
                                 # finally do the copy
                                 uv_other.data.foreach_set("uv", uv_array)
+                                mesh_other.update()
 
         if is_editmode:
             bpy.ops.object.mode_set(mode='EDIT', toggle=False)
@@ -547,12 +586,15 @@ class JoinUVs(Operator):
 
 
 class MakeDupliFace(Operator):
-    """Make linked objects into dupli-faces"""
+    """Convert objects into instanced faces"""
     bl_idname = "object.make_dupli_face"
-    bl_label = "Make Dupli-Face"
+    bl_label = "Make Instance Face"
+    bl_options = {'REGISTER', 'UNDO'}
 
-    def _main(self, context):
+    @staticmethod
+    def _main(context):
         from mathutils import Vector
+        from collections import defaultdict
 
         SCALE_FAC = 0.01
         offset = 0.5 * SCALE_FAC
@@ -567,13 +609,11 @@ class MakeDupliFace(Operator):
             trans = matrix.to_translation()
             rot = matrix.to_3x3()  # also contains scale
 
-            return [(rot * b) + trans for b in base_tri]
-        scene = context.scene
-        linked = {}
+            return [(rot @ b) + trans for b in base_tri]
+        linked = defaultdict(list)
         for obj in context.selected_objects:
-            data = obj.data
-            if data:
-                linked.setdefault(data, []).append(obj)
+            if obj.type == 'MESH':
+                linked[obj.data].append(obj)
 
         for data, objects in linked.items():
             face_verts = [axis for obj in objects
@@ -596,24 +636,23 @@ class MakeDupliFace(Operator):
             mesh.polygons.foreach_set("loop_total", (4,) * nbr_faces)
             mesh.update()  # generates edge data
 
-            # pick an object to use
-            obj = objects[0]
-
             ob_new = bpy.data.objects.new(mesh.name, mesh)
-            base = scene.objects.link(ob_new)
-            base.layers[:] = obj.layers
+            context.collection.objects.link(ob_new)
 
             ob_inst = bpy.data.objects.new(data.name, data)
-            base = scene.objects.link(ob_inst)
-            base.layers[:] = obj.layers
+            context.collection.objects.link(ob_inst)
+
+            ob_new.instance_type = 'FACES'
+            ob_inst.parent = ob_new
+            ob_new.use_instance_faces_scale = True
+            ob_new.instance_faces_scale = 1.0 / SCALE_FAC
+
+            ob_inst.select_set(True)
+            ob_new.select_set(True)
 
             for obj in objects:
-                scene.objects.unlink(obj)
-
-            ob_new.dupli_type = 'FACES'
-            ob_inst.parent = ob_new
-            ob_new.use_dupli_faces_scale = True
-            ob_new.dupli_faces_scale = 1.0 / SCALE_FAC
+                for collection in obj.users_collection:
+                    collection.objects.unlink(obj)
 
     def execute(self, context):
         self._main(context)
@@ -622,7 +661,7 @@ class MakeDupliFace(Operator):
 
 class IsolateTypeRender(Operator):
     """Hide unselected render objects of same type as active """ \
-    """by setting the hide render flag"""
+        """by setting the hide render flag"""
     bl_idname = "object.isolate_type_render"
     bl_label = "Restrict Render Unselected"
     bl_options = {'REGISTER', 'UNDO'}
@@ -632,7 +671,7 @@ class IsolateTypeRender(Operator):
 
         for obj in context.visible_objects:
 
-            if obj.select:
+            if obj.select_get():
                 obj.hide_render = False
             else:
                 if obj.type == act_type:
@@ -653,6 +692,82 @@ class ClearAllRestrictRender(Operator):
         return {'FINISHED'}
 
 
+class TransformsToDeltas(Operator):
+    """Convert normal object transforms to delta transforms, """ \
+        """any existing delta transforms will be included as well"""
+    bl_idname = "object.transforms_to_deltas"
+    bl_label = "Transforms to Deltas"
+    bl_options = {'REGISTER', 'UNDO'}
+
+    mode: EnumProperty(
+        items=(
+            ('ALL', "All Transforms", "Transfer location, rotation, and scale transforms"),
+            ('LOC', "Location", "Transfer location transforms only"),
+            ('ROT', "Rotation", "Transfer rotation transforms only"),
+            ('SCALE', "Scale", "Transfer scale transforms only"),
+        ),
+        name="Mode",
+        description="Which transforms to transfer",
+        default='ALL',
+    )
+    reset_values: BoolProperty(
+        name="Reset Values",
+        description=("Clear transform values after transferring to deltas"),
+        default=True,
+    )
+
+    @classmethod
+    def poll(cls, context):
+        obs = context.selected_editable_objects
+        return (obs is not None)
+
+    def execute(self, context):
+        for obj in context.selected_editable_objects:
+            if self.mode in {'ALL', 'LOC'}:
+                self.transfer_location(obj)
+
+            if self.mode in {'ALL', 'ROT'}:
+                self.transfer_rotation(obj)
+
+            if self.mode in {'ALL', 'SCALE'}:
+                self.transfer_scale(obj)
+
+        return {'FINISHED'}
+
+    def transfer_location(self, obj):
+        obj.delta_location += obj.location
+
+        if self.reset_values:
+            obj.location.zero()
+
+    def transfer_rotation(self, obj):
+        # TODO: add transforms together...
+        if obj.rotation_mode == 'QUATERNION':
+            delta = obj.delta_rotation_quaternion.copy()
+            obj.delta_rotation_quaternion = obj.rotation_quaternion
+            obj.delta_rotation_quaternion.rotate(delta)
+
+            if self.reset_values:
+                obj.rotation_quaternion.identity()
+        elif obj.rotation_mode == 'AXIS_ANGLE':
+            pass  # Unsupported
+        else:
+            delta = obj.delta_rotation_euler.copy()
+            obj.delta_rotation_euler = obj.rotation_euler
+            obj.delta_rotation_euler.rotate(delta)
+
+            if self.reset_values:
+                obj.rotation_euler.zero()
+
+    def transfer_scale(self, obj):
+        obj.delta_scale[0] *= obj.scale[0]
+        obj.delta_scale[1] *= obj.scale[1]
+        obj.delta_scale[2] *= obj.scale[2]
+
+        if self.reset_values:
+            obj.scale[:] = (1, 1, 1)
+
+
 class TransformsToDeltasAnim(Operator):
     """Convert object animation for normal transforms to delta transforms"""
     bl_idname = "object.anim_transforms_to_deltas"
@@ -665,8 +780,18 @@ class TransformsToDeltasAnim(Operator):
         return (obs is not None)
 
     def execute(self, context):
+        # map from standard transform paths to "new" transform paths
+        STANDARD_TO_DELTA_PATHS = {
+            "location": "delta_location",
+            "rotation_euler": "delta_rotation_euler",
+            "rotation_quaternion": "delta_rotation_quaternion",
+            # "rotation_axis_angle" : "delta_rotation_axis_angle",
+            "scale": "delta_scale"
+        }
+        DELTA_PATHS = STANDARD_TO_DELTA_PATHS.values()
+
+        # try to apply on each selected object
         for obj in context.selected_editable_objects:
-            # get animation data
             adt = obj.animation_data
             if (adt is None) or (adt.action is None):
                 self.report({'WARNING'},
@@ -674,6 +799,40 @@ class TransformsToDeltasAnim(Operator):
                             obj.name)
                 continue
 
+            # first pass over F-Curves: ensure that we don't have conflicting
+            # transforms already (e.g. if this was applied already) T29110.
+            existingFCurves = {}
+            for fcu in adt.action.fcurves:
+                # get "delta" path - i.e. the final paths which may clash
+                path = fcu.data_path
+                if path in STANDARD_TO_DELTA_PATHS:
+                    # to be converted - conflicts may exist...
+                    dpath = STANDARD_TO_DELTA_PATHS[path]
+                elif path in DELTA_PATHS:
+                    # already delta - check for conflicts...
+                    dpath = path
+                else:
+                    # non-transform - ignore
+                    continue
+
+                # a delta path like this for the same index shouldn't
+                # exist already, otherwise we've got a conflict
+                if dpath in existingFCurves:
+                    # ensure that this index hasn't occurred before
+                    if fcu.array_index in existingFCurves[dpath]:
+                        # conflict
+                        self.report({'ERROR'},
+                                    "Object '%r' already has '%r' F-Curve(s). "
+                                    "Remove these before trying again" %
+                                    (obj.name, dpath))
+                        return {'CANCELLED'}
+                    else:
+                        # no conflict here
+                        existingFCurves[dpath] += [fcu.array_index]
+                else:
+                    # no conflict yet
+                    existingFCurves[dpath] = [fcu.array_index]
+
             # if F-Curve uses standard transform path
             # just append "delta_" to this path
             for fcu in adt.action.fcurves:
@@ -687,8 +846,8 @@ class TransformsToDeltasAnim(Operator):
                     fcu.data_path = "delta_rotation_quaternion"
                     obj.rotation_quaternion.identity()
                 # XXX: currently not implemented
-                #~ elif fcu.data_path == "rotation_axis_angle":
-                #~    fcu.data_path = "delta_rotation_axis_angle"
+                # ~ elif fcu.data_path == "rotation_axis_angle":
+                # ~    fcu.data_path = "delta_rotation_axis_angle"
                 elif fcu.data_path == "scale":
                     fcu.data_path = "delta_scale"
                     obj.scale = 1.0, 1.0, 1.0
@@ -700,26 +859,156 @@ class TransformsToDeltasAnim(Operator):
 
 
 class DupliOffsetFromCursor(Operator):
-    """Set offset used for DupliGroup based on cursor position"""
-    bl_idname = "object.dupli_offset_from_cursor"
-    bl_label = "Set Offset From Cursor"
-    bl_options = {'REGISTER', 'UNDO'}
-
-    group = IntProperty(
-            name="Group",
-            description="Group index to set offset for",
-            default=0,
-            )
+    """Set offset used for collection instances based on cursor position"""
+    bl_idname = "object.instance_offset_from_cursor"
+    bl_label = "Set Offset from Cursor"
+    bl_options = {'INTERNAL', 'UNDO'}
 
     @classmethod
     def poll(cls, context):
-        return  context.active_object is not None
+        return (context.active_object is not None)
 
     def execute(self, context):
         scene = context.scene
-        ob = context.active_object
-        group = self.group
+        collection = context.collection
 
-        ob.users_group[group].dupli_offset = scene.cursor_location
+        collection.instance_offset = scene.cursor.location
 
         return {'FINISHED'}
+
+
+class LoadImageAsEmpty:
+    bl_options = {'REGISTER', 'UNDO'}
+
+    filepath: StringProperty(
+        subtype='FILE_PATH'
+    )
+
+    filter_image: BoolProperty(default=True, options={'HIDDEN', 'SKIP_SAVE'})
+    filter_folder: BoolProperty(default=True, options={'HIDDEN', 'SKIP_SAVE'})
+
+    view_align: BoolProperty(
+        name="Align to View",
+        default=True,
+    )
+
+    @classmethod
+    def poll(cls, context):
+        return context.mode == 'OBJECT'
+
+    def invoke(self, context, _event):
+        context.window_manager.fileselect_add(self)
+        return {'RUNNING_MODAL'}
+
+    def execute(self, context):
+        scene = context.scene
+        cursor = scene.cursor.location
+
+        try:
+            image = bpy.data.images.load(self.filepath, check_existing=True)
+        except RuntimeError as ex:
+            self.report({'ERROR'}, str(ex))
+            return {'CANCELLED'}
+
+        bpy.ops.object.empty_add(
+            'INVOKE_REGION_WIN',
+            type='IMAGE',
+            location=cursor,
+            align=('VIEW' if self.view_align else 'WORLD'),
+        )
+
+        view_layer = context.view_layer
+        obj = view_layer.objects.active
+        obj.data = image
+        obj.empty_display_size = 5.0
+        self.set_settings(context, obj)
+        return {'FINISHED'}
+
+    def set_settings(self, context, obj):
+        pass
+
+
+class LoadBackgroundImage(LoadImageAsEmpty, Operator):
+    """Add a reference image into the background behind objects"""
+    bl_idname = "object.load_background_image"
+    bl_label = "Load Background Image"
+
+    def set_settings(self, context, obj):
+        obj.empty_image_depth = 'BACK'
+        obj.empty_image_side = 'FRONT'
+
+        if context.space_data.type == 'VIEW_3D':
+            if not context.space_data.region_3d.is_perspective:
+                obj.show_empty_image_perspective = False
+
+
+class LoadReferenceImage(LoadImageAsEmpty, Operator):
+    """Add a reference image into the scene between objects"""
+    bl_idname = "object.load_reference_image"
+    bl_label = "Load Reference Image"
+
+    def set_settings(self, context, obj):
+        pass
+
+
+class OBJECT_OT_assign_property_defaults(Operator):
+    """Assign the current values of custom properties as their defaults, """ \
+    """for use as part of the rest pose state in NLA track mixing"""
+    bl_idname = "object.assign_property_defaults"
+    bl_label = "Assign Custom Property Values as Default"
+    bl_options = {'UNDO', 'REGISTER'}
+
+    process_data: BoolProperty(name="Process data properties", default=True)
+    process_bones: BoolProperty(name="Process bone properties", default=True)
+
+    @classmethod
+    def poll(cls, context):
+        obj = context.active_object
+        return obj is not None and obj.library is None and obj.mode in {'POSE', 'OBJECT'}
+
+    @staticmethod
+    def assign_defaults(obj):
+        from rna_prop_ui import rna_idprop_ui_prop_default_set
+
+        rna_properties = {'_RNA_UI'} | {prop.identifier for prop in obj.bl_rna.properties if prop.is_runtime}
+
+        for prop, value in obj.items():
+            if prop not in rna_properties:
+                rna_idprop_ui_prop_default_set(obj, prop, value)
+
+    def execute(self, context):
+        obj = context.active_object
+
+        self.assign_defaults(obj)
+
+        if self.process_bones and obj.pose:
+            for pbone in obj.pose.bones:
+                self.assign_defaults(pbone)
+
+        if self.process_data and obj.data and obj.data.library is None:
+            self.assign_defaults(obj.data)
+
+            if self.process_bones and isinstance(obj.data, bpy.types.Armature):
+                for bone in obj.data.bones:
+                    self.assign_defaults(bone)
+
+        return {'FINISHED'}
+
+
+classes = (
+    ClearAllRestrictRender,
+    DupliOffsetFromCursor,
+    IsolateTypeRender,
+    JoinUVs,
+    LoadBackgroundImage,
+    LoadReferenceImage,
+    MakeDupliFace,
+    SelectCamera,
+    SelectHierarchy,
+    SelectPattern,
+    ShapeTransfer,
+    SubdivisionSet,
+    TransformsToDeltas,
+    TransformsToDeltasAnim,
+    OBJECT_OT_assign_property_defaults,
+)
